package com.sun.corba.se.impl.protocol.giopmsgheaders;

import com.sun.corba.se.impl.orbutil.ORBUtility;
import sun.corba.SharedSecrets;
import org.omg.CORBA.SystemException;
import org.omg.IOP.TaggedProfile;
import com.sun.corba.se.spi.ior.iiop.IIOPFactories;
import com.sun.corba.se.impl.protocol.AddressingDispositionException;
import com.sun.corba.se.spi.ior.ObjectKey;
import java.util.Iterator;
import com.sun.corba.se.spi.ior.iiop.IIOPProfile;
import com.sun.corba.se.spi.ior.iiop.RequestPartitioningComponent;
import com.sun.corba.se.spi.ior.iiop.IIOPProfileTemplate;
import com.sun.corba.se.spi.ior.IOR;
import org.omg.CORBA.Principal;
import com.sun.corba.se.spi.servicecontext.ServiceContexts;
import com.sun.corba.se.spi.transport.ReadTimeouts;
import com.sun.corba.se.spi.ior.iiop.GIOPVersion;
import org.omg.CORBA.CompletionStatus;
import com.sun.corba.se.impl.encoding.CDRInputStream_1_0;
import com.sun.corba.se.impl.encoding.ByteBufferWithInfo;
import java.io.IOException;
import com.sun.corba.se.spi.transport.CorbaConnection;
import com.sun.corba.se.spi.orb.ORB;
import com.sun.corba.se.impl.logging.ORBUtilSystemException;
import java.nio.ByteBuffer;

public abstract class MessageBase implements Message
{
    public byte[] giopHeader;
    private ByteBuffer byteBuffer;
    private int threadPoolToUse;
    byte encodingVersion;
    private static ORBUtilSystemException wrapper;
    
    public MessageBase() {
        this.encodingVersion = 0;
    }
    
    public static String typeToString(int n) {
        return typeToString((byte)n);
    }
    
    public static String typeToString(byte i) {
        String string = (int)i + "/";
        String s;
        switch (i) {
            case 0: {
                s = string + "GIOPRequest";
                break;
            }
            case 1: {
                s = string + "GIOPReply";
                break;
            }
            case 2: {
                s = string + "GIOPCancelRequest";
                break;
            }
            case 3: {
                s = string + "GIOPLocateRequest";
                break;
            }
            case 4: {
                s = string + "GIOPLocateReply";
                break;
            }
            case 5: {
                s = string + "GIOPCloseConnection";
                break;
            }
            case 6: {
                s = string + "GIOPMessageError";
                break;
            }
            case 7: {
                s = string + "GIOPFragment";
                break;
            }
            default: {
                s = string + "Unknown";
                break;
            }
        }
        return s;
    }
    
    public static MessageBase readGIOPMessage(ORB orb, CorbaConnection corbaConnection) {
        return (MessageBase)readGIOPBody(orb, corbaConnection, readGIOPHeader(orb, corbaConnection));
    }
    
    public static MessageBase readGIOPHeader(ORB orb, CorbaConnection corbaConnection) {
        Object o = null;
        ReadTimeouts transportTCPReadTimeouts = orb.getORBData().getTransportTCPReadTimeouts();
        ByteBuffer read;
        try {
            read = corbaConnection.read(12, 0, 12, (long)transportTCPReadTimeouts.get_max_giop_header_time_to_wait());
        }
        catch (IOException ex) {
            throw MessageBase.wrapper.ioexceptionWhenReadingConnection(ex);
        }
        if (orb.giopDebugFlag) {
            dprint(".readGIOPHeader: " + typeToString(read.get(7)));
            dprint(".readGIOPHeader: GIOP header is: ");
            ByteBuffer readOnlyBuffer = read.asReadOnlyBuffer();
            readOnlyBuffer.position(0).limit(12);
            ByteBufferWithInfo byteBufferWithInfo = new ByteBufferWithInfo(orb, readOnlyBuffer);
            byteBufferWithInfo.buflen = 12;
            CDRInputStream_1_0.printBuffer(byteBufferWithInfo);
        }
        int n = (read.get(0) << 24 & 0xFF000000) | (read.get(1) << 16 & 0xFF0000) | (read.get(2) << 8 & 0xFF00) | (read.get(3) << 0 & 0xFF);
        if (n != 1195986768) {
            throw MessageBase.wrapper.giopMagicError(CompletionStatus.COMPLETED_MAYBE);
        }
        int value = 0;
        if (read.get(4) == 13 && read.get(5) <= 1 && read.get(5) > 0 && orb.getORBData().isJavaSerializationEnabled()) {
            value = read.get(5);
            read.put(4, (byte)1);
            read.put(5, (byte)2);
        }
        GIOPVersion giopVersion = orb.getORBData().getGIOPVersion();
        if (orb.giopDebugFlag) {
            dprint(".readGIOPHeader: Message GIOP version: " + (int)read.get(4) + 46 + (int)read.get(5));
            dprint(".readGIOPHeader: ORB Max GIOP Version: " + giopVersion);
        }
        if ((read.get(4) > giopVersion.getMajor() || (read.get(4) == giopVersion.getMajor() && read.get(5) > giopVersion.getMinor())) && read.get(7) != 6) {
            throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
        }
        AreFragmentsAllowed(read.get(4), read.get(5), read.get(6), read.get(7));
        switch (read.get(7)) {
            case 0: {
                if (orb.giopDebugFlag) {
                    dprint(".readGIOPHeader: creating RequestMessage");
                }
                if (read.get(4) == 1 && read.get(5) == 0) {
                    o = new RequestMessage_1_0(orb);
                    break;
                }
                if (read.get(4) == 1 && read.get(5) == 1) {
                    o = new RequestMessage_1_1(orb);
                    break;
                }
                if (read.get(4) == 1 && read.get(5) == 2) {
                    o = new RequestMessage_1_2(orb);
                    break;
                }
                throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
            }
            case 3: {
                if (orb.giopDebugFlag) {
                    dprint(".readGIOPHeader: creating LocateRequestMessage");
                }
                if (read.get(4) == 1 && read.get(5) == 0) {
                    o = new LocateRequestMessage_1_0(orb);
                    break;
                }
                if (read.get(4) == 1 && read.get(5) == 1) {
                    o = new LocateRequestMessage_1_1(orb);
                    break;
                }
                if (read.get(4) == 1 && read.get(5) == 2) {
                    o = new LocateRequestMessage_1_2(orb);
                    break;
                }
                throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
            }
            case 2: {
                if (orb.giopDebugFlag) {
                    dprint(".readGIOPHeader: creating CancelRequestMessage");
                }
                if (read.get(4) == 1 && read.get(5) == 0) {
                    o = new CancelRequestMessage_1_0();
                    break;
                }
                if (read.get(4) == 1 && read.get(5) == 1) {
                    o = new CancelRequestMessage_1_1();
                    break;
                }
                if (read.get(4) == 1 && read.get(5) == 2) {
                    o = new CancelRequestMessage_1_2();
                    break;
                }
                throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
            }
            case 1: {
                if (orb.giopDebugFlag) {
                    dprint(".readGIOPHeader: creating ReplyMessage");
                }
                if (read.get(4) == 1 && read.get(5) == 0) {
                    o = new ReplyMessage_1_0(orb);
                    break;
                }
                if (read.get(4) == 1 && read.get(5) == 1) {
                    o = new ReplyMessage_1_1(orb);
                    break;
                }
                if (read.get(4) == 1 && read.get(5) == 2) {
                    o = new ReplyMessage_1_2(orb);
                    break;
                }
                throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
            }
            case 4: {
                if (orb.giopDebugFlag) {
                    dprint(".readGIOPHeader: creating LocateReplyMessage");
                }
                if (read.get(4) == 1 && read.get(5) == 0) {
                    o = new LocateReplyMessage_1_0(orb);
                    break;
                }
                if (read.get(4) == 1 && read.get(5) == 1) {
                    o = new LocateReplyMessage_1_1(orb);
                    break;
                }
                if (read.get(4) == 1 && read.get(5) == 2) {
                    o = new LocateReplyMessage_1_2(orb);
                    break;
                }
                throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
            }
            case 5:
            case 6: {
                if (orb.giopDebugFlag) {
                    dprint(".readGIOPHeader: creating Message for CloseConnection or MessageError");
                }
                if (read.get(4) == 1 && read.get(5) == 0) {
                    o = new Message_1_0();
                    break;
                }
                if (read.get(4) == 1 && read.get(5) == 1) {
                    o = new Message_1_1();
                    break;
                }
                if (read.get(4) == 1 && read.get(5) == 2) {
                    o = new Message_1_1();
                    break;
                }
                throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
            }
            case 7: {
                if (orb.giopDebugFlag) {
                    dprint(".readGIOPHeader: creating FragmentMessage");
                }
                if (read.get(4) == 1 && read.get(5) == 0) {
                    break;
                }
                if (read.get(4) == 1 && read.get(5) == 1) {
                    o = new FragmentMessage_1_1();
                    break;
                }
                if (read.get(4) == 1 && read.get(5) == 2) {
                    o = new FragmentMessage_1_2();
                    break;
                }
                throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
            }
            default: {
                if (orb.giopDebugFlag) {
                    dprint(".readGIOPHeader: UNKNOWN MESSAGE TYPE: " + (int)read.get(7));
                }
                throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
            }
        }
        if (read.get(4) == 1 && read.get(5) == 0) {
            Message_1_0 message_1_2 = (Message_1_0)o;
            message_1_2.magic = n;
            message_1_2.GIOP_version = new GIOPVersion(read.get(4), read.get(5));
            message_1_2.byte_order = (read.get(6) == 1);
            o.threadPoolToUse = 0;
            message_1_2.message_type = read.get(7);
            message_1_2.message_size = readSize(read.get(8), read.get(9), read.get(10), read.get(11), message_1_2.isLittleEndian()) + 12;
        }
        else {
            Message_1_1 message_1_1 = (Message_1_1)o;
            message_1_1.magic = n;
            message_1_1.GIOP_version = new GIOPVersion(read.get(4), read.get(5));
            message_1_1.flags = (byte)(read.get(6) & 0x3);
            o.threadPoolToUse = (read.get(6) >>> 2 & 0x3F);
            message_1_1.message_type = read.get(7);
            message_1_1.message_size = readSize(read.get(8), read.get(9), read.get(10), read.get(11), message_1_1.isLittleEndian()) + 12;
        }
        if (orb.giopDebugFlag) {
            dprint(".readGIOPHeader: header construction complete.");
            ByteBuffer readOnlyBuffer2 = read.asReadOnlyBuffer();
            byte[] array = new byte[12];
            readOnlyBuffer2.position(0).limit(12);
            readOnlyBuffer2.get(array, 0, array.length);
            o.giopHeader = array;
        }
        o.setByteBuffer(read);
        o.setEncodingVersion((byte)value);
        return o;
    }
    
    public static Message readGIOPBody(ORB orb, CorbaConnection corbaConnection, Message message) {
        ReadTimeouts transportTCPReadTimeouts = orb.getORBData().getTransportTCPReadTimeouts();
        ByteBuffer byteBuffer = message.getByteBuffer();
        byteBuffer.position(12);
        int n = message.getSize() - 12;
        ByteBuffer read;
        try {
            read = corbaConnection.read(byteBuffer, 12, n, (long)transportTCPReadTimeouts.get_max_time_to_wait());
        }
        catch (IOException ex) {
            throw MessageBase.wrapper.ioexceptionWhenReadingConnection(ex);
        }
        message.setByteBuffer(read);
        if (orb.giopDebugFlag) {
            dprint(".readGIOPBody: received message:");
            ByteBuffer readOnlyBuffer = read.asReadOnlyBuffer();
            readOnlyBuffer.position(0).limit(message.getSize());
            CDRInputStream_1_0.printBuffer(new ByteBufferWithInfo(orb, readOnlyBuffer));
            return message;
        }
        return message;
    }
    
    private static RequestMessage createRequest(ORB orb, GIOPVersion giopVersion, byte encodingVersion, int n, boolean b, byte[] array, String s, ServiceContexts serviceContexts, Principal principal) {
        if (giopVersion.equals(GIOPVersion.V1_0)) {
            return new RequestMessage_1_0(orb, serviceContexts, n, b, array, s, principal);
        }
        if (giopVersion.equals(GIOPVersion.V1_1)) {
            return new RequestMessage_1_1(orb, serviceContexts, n, b, new byte[] { 0, 0, 0 }, array, s, principal);
        }
        if (giopVersion.equals(GIOPVersion.V1_2)) {
            int n2;
            if (b) {
                n2 = 3;
            }
            else {
                n2 = 0;
            }
            TargetAddress o = new TargetAddress();
            o.object_key(array);
            RequestMessage_1_2 o2 = new RequestMessage_1_2(orb, n, (byte)n2, new byte[] { 0, 0, 0 }, o, s, serviceContexts);
            o2.setEncodingVersion(encodingVersion);
            return o2;
        }
        else {
            throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
        }
    }
    
    public static RequestMessage createRequest(ORB orb, GIOPVersion giopVersion, byte encodingVersion, int n, boolean b, IOR ior, short n2, String s, ServiceContexts serviceContexts, Principal principal) {
        IIOPProfile iiopProfile = ior.getProfile();
        RequestMessage request;
        if (n2 == 0) {
            iiopProfile = ior.getProfile();
            request = createRequest(orb, giopVersion, encodingVersion, n, b, iiopProfile.getObjectKey().getBytes(orb), s, serviceContexts, principal);
        }
        else {
            if (!giopVersion.equals(GIOPVersion.V1_2)) {
                throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
            }
            int n3;
            if (b) {
                n3 = 3;
            }
            else {
                n3 = 0;
            }
            TargetAddress o = new TargetAddress();
            if (n2 == 1) {
                iiopProfile = ior.getProfile();
                o.profile(iiopProfile.getIOPProfile());
            }
            else if (n2 == 2) {
                o.ior(new IORAddressingInfo(0, ior.getIOPIOR()));
            }
            else {
                throw MessageBase.wrapper.illegalTargetAddressDisposition(CompletionStatus.COMPLETED_NO);
            }
            request = new RequestMessage_1_2(orb, n, (byte)n3, new byte[] { 0, 0, 0 }, o, s, serviceContexts);
            request.setEncodingVersion(encodingVersion);
        }
        if (giopVersion.supportsIORIIOPProfileComponents()) {
            int requestPartitioningId = 0;
            Iterator<Object> iteratorById = ((IIOPProfileTemplate)iiopProfile.getTaggedProfileTemplate()).iteratorById(1398099457);
            if (iteratorById.hasNext()) {
                requestPartitioningId = ((RequestPartitioningComponent)iteratorById.next()).getRequestPartitioningId();
            }
            if (requestPartitioningId < 0 || requestPartitioningId > 63) {
                throw MessageBase.wrapper.invalidRequestPartitioningId(new Integer(requestPartitioningId), new Integer(0), new Integer(63));
            }
            request.setThreadPoolToUse(requestPartitioningId);
        }
        return request;
    }
    
    public static ReplyMessage createReply(ORB orb, GIOPVersion giopVersion, byte encodingVersion, int n, int n2, ServiceContexts serviceContexts, IOR ior) {
        if (giopVersion.equals(GIOPVersion.V1_0)) {
            return new ReplyMessage_1_0(orb, serviceContexts, n, n2, ior);
        }
        if (giopVersion.equals(GIOPVersion.V1_1)) {
            return new ReplyMessage_1_1(orb, serviceContexts, n, n2, ior);
        }
        if (giopVersion.equals(GIOPVersion.V1_2)) {
            ReplyMessage_1_2 replyMessage_1_2 = new ReplyMessage_1_2(orb, n, n2, serviceContexts, ior);
            replyMessage_1_2.setEncodingVersion(encodingVersion);
            return replyMessage_1_2;
        }
        throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
    }
    
    public static LocateRequestMessage createLocateRequest(ORB orb, GIOPVersion giopVersion, byte encodingVersion, int n, byte[] array) {
        if (giopVersion.equals(GIOPVersion.V1_0)) {
            return new LocateRequestMessage_1_0(orb, n, array);
        }
        if (giopVersion.equals(GIOPVersion.V1_1)) {
            return new LocateRequestMessage_1_1(orb, n, array);
        }
        if (giopVersion.equals(GIOPVersion.V1_2)) {
            TargetAddress targetAddress = new TargetAddress();
            targetAddress.object_key(array);
            LocateRequestMessage_1_2 locateRequestMessage_1_2 = new LocateRequestMessage_1_2(orb, n, targetAddress);
            locateRequestMessage_1_2.setEncodingVersion(encodingVersion);
            return locateRequestMessage_1_2;
        }
        throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
    }
    
    public static LocateReplyMessage createLocateReply(ORB orb, GIOPVersion giopVersion, byte encodingVersion, int n, int n2, IOR ior) {
        if (giopVersion.equals(GIOPVersion.V1_0)) {
            return new LocateReplyMessage_1_0(orb, n, n2, ior);
        }
        if (giopVersion.equals(GIOPVersion.V1_1)) {
            return new LocateReplyMessage_1_1(orb, n, n2, ior);
        }
        if (giopVersion.equals(GIOPVersion.V1_2)) {
            LocateReplyMessage_1_2 locateReplyMessage_1_2 = new LocateReplyMessage_1_2(orb, n, n2, ior);
            locateReplyMessage_1_2.setEncodingVersion(encodingVersion);
            return locateReplyMessage_1_2;
        }
        throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
    }
    
    public static CancelRequestMessage createCancelRequest(GIOPVersion giopVersion, int n) {
        if (giopVersion.equals(GIOPVersion.V1_0)) {
            return new CancelRequestMessage_1_0(n);
        }
        if (giopVersion.equals(GIOPVersion.V1_1)) {
            return new CancelRequestMessage_1_1(n);
        }
        if (giopVersion.equals(GIOPVersion.V1_2)) {
            return new CancelRequestMessage_1_2(n);
        }
        throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
    }
    
    public static Message createCloseConnection(GIOPVersion giopVersion) {
        if (giopVersion.equals(GIOPVersion.V1_0)) {
            return new Message_1_0(1195986768, false, (byte)5, 0);
        }
        if (giopVersion.equals(GIOPVersion.V1_1)) {
            return new Message_1_1(1195986768, GIOPVersion.V1_1, (byte)0, (byte)5, 0);
        }
        if (giopVersion.equals(GIOPVersion.V1_2)) {
            return new Message_1_1(1195986768, GIOPVersion.V1_2, (byte)0, (byte)5, 0);
        }
        throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
    }
    
    public static Message createMessageError(GIOPVersion giopVersion) {
        if (giopVersion.equals(GIOPVersion.V1_0)) {
            return new Message_1_0(1195986768, false, (byte)6, 0);
        }
        if (giopVersion.equals(GIOPVersion.V1_1)) {
            return new Message_1_1(1195986768, GIOPVersion.V1_1, (byte)0, (byte)6, 0);
        }
        if (giopVersion.equals(GIOPVersion.V1_2)) {
            return new Message_1_1(1195986768, GIOPVersion.V1_2, (byte)0, (byte)6, 0);
        }
        throw MessageBase.wrapper.giopVersionError(CompletionStatus.COMPLETED_MAYBE);
    }
    
    public static FragmentMessage createFragmentMessage(GIOPVersion giopVersion) {
        return null;
    }
    
    public static int getRequestId(Message message) {
        switch (message.getType()) {
            case 0: {
                return ((RequestMessage)message).getRequestId();
            }
            case 1: {
                return ((ReplyMessage)message).getRequestId();
            }
            case 3: {
                return ((LocateRequestMessage)message).getRequestId();
            }
            case 4: {
                return ((LocateReplyMessage)message).getRequestId();
            }
            case 2: {
                return ((CancelRequestMessage)message).getRequestId();
            }
            case 7: {
                return ((FragmentMessage)message).getRequestId();
            }
            default: {
                throw MessageBase.wrapper.illegalGiopMsgType(CompletionStatus.COMPLETED_MAYBE);
            }
        }
    }
    
    public static void setFlag(ByteBuffer byteBuffer, int n) {
        byteBuffer.put(6, (byte)(byteBuffer.get(6) | n));
    }
    
    public static void clearFlag(byte[] array, int n) {
        int n2 = 6;
        array[n2] &= (0xFF ^ n);
    }
    
    private static void AreFragmentsAllowed(byte b, byte b2, byte b3, byte b4) {
        if (b == 1 && b2 == 0 && b4 == 7) {
            throw MessageBase.wrapper.fragmentationDisallowed(CompletionStatus.COMPLETED_MAYBE);
        }
        if ((b3 & 0x2) == 0x2) {
            switch (b4) {
                case 2:
                case 5:
                case 6: {
                    throw MessageBase.wrapper.fragmentationDisallowed(CompletionStatus.COMPLETED_MAYBE);
                }
                case 3:
                case 4: {
                    if (b == 1 && b2 == 1) {
                        throw MessageBase.wrapper.fragmentationDisallowed(CompletionStatus.COMPLETED_MAYBE);
                    }
                    break;
                }
            }
        }
        return;
    }
    
    static ObjectKey extractObjectKey(byte[] array, ORB orb) {
        try {
            if (array != null) {
                ObjectKey create = orb.getObjectKeyFactory().create(array);
                if (create != null) {
                    return create;
                }
            }
        }
        catch (Exception ex) {}
        throw MessageBase.wrapper.invalidObjectKey();
    }
    
    static ObjectKey extractObjectKey(TargetAddress targetAddress, ORB orb) {
        int giopTargetAddressPreference = orb.getORBData().getGIOPTargetAddressPreference();
        int discriminator = targetAddress.discriminator();
        switch (giopTargetAddressPreference) {
            case 0: {
                if (discriminator != 0) {
                    throw new AddressingDispositionException((short)0);
                }
                break;
            }
            case 1: {
                if (discriminator != 1) {
                    throw new AddressingDispositionException((short)1);
                }
                break;
            }
            case 2: {
                if (discriminator != 2) {
                    throw new AddressingDispositionException((short)2);
                }
            }
            case 3: {
                break;
            }
            default: {
                throw MessageBase.wrapper.orbTargetAddrPreferenceInExtractObjectkeyInvalid();
            }
        }
        try {
            switch (discriminator) {
                case 0: {
                    byte[] object_key = targetAddress.object_key();
                    if (object_key != null) {
                        ObjectKey create = orb.getObjectKeyFactory().create(object_key);
                        if (create != null) {
                            return create;
                        }
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 1: {
                    TaggedProfile profile = targetAddress.profile();
                    if (profile != null) {
                        ObjectKey objectKey = IIOPFactories.makeIIOPProfile(orb, profile).getObjectKey();
                        if (objectKey != null) {
                            return objectKey;
                        }
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 2: {
                    IORAddressingInfo ior = targetAddress.ior();
                    if (ior != null) {
                        ObjectKey objectKey2 = IIOPFactories.makeIIOPProfile(orb, ior.ior.profiles[ior.selected_profile_index]).getObjectKey();
                        if (objectKey2 != null) {
                            return objectKey2;
                        }
                    }
                    else {
                        break;
                    }
                }
            }
        }
        catch (Exception ex) {}
        throw MessageBase.wrapper.invalidObjectKey();
    }
    
    private static int readSize(byte b, byte b2, byte b3, byte b4, boolean b5) {
        int n;
        int n2;
        int n3;
        int n4;
        if (!b5) {
            n = (b << 24 & 0xFF000000);
            n2 = (b2 << 16 & 0xFF0000);
            n3 = (b3 << 8 & 0xFF00);
            n4 = (b4 << 0 & 0xFF);
        }
        else {
            n = (b4 << 24 & 0xFF000000);
            n2 = (b3 << 16 & 0xFF0000);
            n3 = (b2 << 8 & 0xFF00);
            n4 = (b << 0 & 0xFF);
        }
        return n | n2 | n3 | n4;
    }
    
    static void nullCheck(Object o) {
        if (o == null) {
            throw MessageBase.wrapper.nullNotAllowed();
        }
        return;
    }
    
    static SystemException getSystemException(String s, int minor, CompletionStatus completed, String s2, ORBUtilSystemException ex) {
        SystemException ex2;
        try {
            Class<Object> loadClass = SharedSecrets.getJavaCorbaAccess().loadClass(s);
            if (s2 == null) {
                ex2 = (SystemException)loadClass.newInstance();
            }
            else {
                ex2 = (SystemException)loadClass.getConstructor(new Class[] { String.class }).newInstance(new Object[] { s2 });
            }
        }
        catch (Exception ex3) {
            throw ex.badSystemExceptionInReply(CompletionStatus.COMPLETED_MAYBE, ex3);
        }
        ex2.minor = minor;
        ex2.completed = completed;
        return ex2;
    }
    
    @Override
    public void callback(MessageHandler messageHandler) throws IOException {
        messageHandler.handleInput(this);
    }
    
    @Override
    public ByteBuffer getByteBuffer() {
        return this.byteBuffer;
    }
    
    @Override
    public void setByteBuffer(ByteBuffer byteBuffer) {
        this.byteBuffer = byteBuffer;
    }
    
    @Override
    public int getThreadPoolToUse() {
        return this.threadPoolToUse;
    }
    
    @Override
    public byte getEncodingVersion() {
        return this.encodingVersion;
    }
    
    @Override
    public void setEncodingVersion(byte encodingVersion) {
        this.encodingVersion = encodingVersion;
    }
    
    private static void dprint(String s) {
        ORBUtility.dprint("MessageBase", s);
    }
    
    static {
        MessageBase.wrapper = ORBUtilSystemException.get("rpc.protocol");
    }
}
